(function() {
/* VARIABLES */
var board = null;
var game = null;
var boardflipped = false;
var promotionSource = null;
var promotionTarget = null;
var disconnected = false;
var autorefresh = true;

/* FUNCTIONS */
function updateToMove() {
	if ((game.turn() === 'b' && board.orientation() === 'white') || (game.turn() === 'w' && board.orientation() === 'black')) {
		$('#toponmove i').show();
		$('#bottomonmove i').hide();
	} else {
       		$('#toponmove i').hide();
		$('#bottomonmove i').show();
	}
}

function updateConfig(data) {
	//boardflipped = data.board._computed.flipped;
	//board.orientation(boardflipped ? 'black' : 'white');
	updateToMove();
	if(data.console.playsounds) {
		$('#switchsounds i').addClass('fa-volume-up');
		$('#switchsounds i').removeClass('fa-volume-mute');
	}
	else {
		$('#switchsounds i').addClass('fa-volume-mute');
		$('#switchsounds i').removeClass('fa-volume-up');
	}
	$('#seconds').val(parseInt(data.engine.goseconds));
	$('#level').val(parseInt(data.engine._computed.level));
}

function downloadAndUpdateConfig() {
	$.get("../api/config", function (data) {
		updateConfig(data);
	});
}

function checkConnection() {
	if(!autorefresh) {
		$('#connection').removeClass("connected");
		$('#connection').removeClass("disconnected");
		$('#connection').addClass("unknown");
		return;
	}
	var ok = false;
	setTimeout(function() {
		if(!ok) {
			$('#connection').removeClass("unknown");
			$('#connection').removeClass("connected");
			$('#connection').addClass("disconnected");
		}
	}, 200);
	$.get("../api/fen", function (fen) {
		if(fen != game.fen()) {
			reload(fen);
		}
                else if(disconnected) {
                        downloadAndUpdateConfig();
                }
		$('#connection').removeClass("disconnected");
		$('#connection').removeClass("unknown");
		$('#connection').addClass("connected");
                disconnected = false;
		ok = true;
	}).fail(function() {
		$('#connection').removeClass("unknown");
		$('#connection').removeClass("connected");
		$('#connection').addClass("disconnected");
		disconnected = true;
	});
}

function reload(fen) {
	game.load(fen);
       	board.position(fen);
	updateStatus();
}

function simpleCommand(cmd, args, donotreload) {
	hidePromotion();
	$.get("../api/command/"+cmd + '?args=' + (args ? args : ""), function () {
        	if(!donotreload) {
       			$.get("../api/fen", reload);
		}
	});
}

function updateStatus() {
	var status = '';

	var moveColor = 'White';
	if (game.turn() === 'b') {
		moveColor = 'Black';
	}
                    
	updateToMove();
	if (game.in_checkmate()) {
		status = 'Check mate!';
	}
	else if (game.in_draw()) {
		status = 'Drawn!';
	}
	else {
		status = '';
		if (game.in_check()) {
			status = 'Check!';
		}
	}
	$('#status').html(status);
	$('#fen').html(game.fen())
}

function hidePromotion() {
	$('#promotionPieces_w').hide();
	$('#promotionPieces_b').hide();
}
            
function promote(piece) {
	hidePromotion();
	var move = game.move({
		from: promotionSource,
		to: promotionTarget,
		promotion: piece
	});

	// illegal move
	if (move === null) {
		return;
	}
	board.position(game.fen());
	$.get("../api/command/move?args=" + promotionSource + promotionTarget + piece, function () {});
	updateStatus();
}


/* EVENTS */

$('#send').on('submit', function(event) {
	event.preventDefault();
	hidePromotion();
       	var cmd = $('#cmd').val();
	$('#cmd').val('');
	var cmdArr = cmd.split(" ",1);
	var args = cmd.substring(cmdArr[0].length);
	$.get("../api/command/"+cmdArr[0]+"?args=" +  args, function () {
		$.get("../api/fen", function (fen) {
 	              reload(fen);
		      downloadAndUpdateConfig();
		});
	});
	return false;
});

/* disconnect */
$('#disconnect').on('click', function() {
	simpleCommand('quit', '', true);
	$('#connection').addClass("disconnected");
	$('#connection').removeClass("connected");
	disconnected = true;
});

/* right menu actions */
$('#new').on('click', function() {
	simpleCommand('new');
});
$('#refresh').on('click', function() {
	simpleCommand('board');
	downloadAndUpdateConfig();
});
$('#autorefresh').on('change', function() {
	if($(this).is(':checked')) {
		autorefresh = true;
	} else {
		autorefresh = false;
	}
	
});
$('#autorefresh').trigger('change');
$('#flip').on('click', function() {
	//simpleCommand('flipboard');
	board.flip();
	boardflipped = !boardflipped;
	//downloadAndUpdateConfig();
	updateToMove();
	
});
$('#switchsounds').on('click', function() {
	simpleCommand('switchsounds');
	downloadAndUpdateConfig();
});
/* bottom naviagion */
$('#backward').on('click', function() {
	simpleCommand('uu');
});
$('#step-backward').on('click', function() {
	simpleCommand('undo');
});
$('#step-forward').on('click', function() {
	simpleCommand('redo');
});
$('#forward').on('click', function() {
	simpleCommand('redo', '1000');
});
$('#play').on('click', function() {
	simpleCommand('play');
	$('#countdown').html($('#seconds').val());
	var handler = setInterval(function() {
		var s = $('#countdown').text();
		if(!s || s == '0') {
			clearInterval(handler);
			$('#countdown').html('');
			return;
		}
		$('#countdown').html(parseInt(s)-1);

	}, 1000);
});
$('#seconds').on('change', function() {
	simpleCommand('seconds', $(this).val());
});
$('#level').on('change', function() {
	simpleCommand('level', $(this).val());
});

/* promotion */
$('#wqueen').on('click', function()  {promote('q');});
$('#wrook').on('click', function()   {promote('r');});
$('#wbishop').on('click', function() {promote('b');});
$('#wknight').on('click', function() {promote('n');});
$('#bqueen').on('click', function()  {promote('q');});
$('#brook').on('click', function()   {promote('r');});
$('#bbishop').on('click', function() {promote('b');});
$('#bknight').on('click', function() {promote('n');});


/* EXECUTE */
            
$.get("../version", function (version) {
	$("#version").html(version);
});


$.get("../api/version", function (version) {
	$("#apiversion").html(version);
});

 
$.get("../api/fen", function (fen) {
                function onDragStart(source, piece, position, orientation) {
                    // do not pick up pieces if the game is over
                    if (game.game_over()) {
                        return false;
                    }
                    // only pick up pieces for the side to move
                    if ((game.turn() === 'w' && piece.search(/^b/) !== -1) ||
                            (game.turn() === 'b' && piece.search(/^w/) !== -1)) {
                        return false;
                    }
                }
                
                function promotionMove(source, target, color) {
                    promotionSource = source;
                    promotionTarget = target;
                    var h = ($('#board').height()-60)/2;
                    $('#promotionPieces_'+color).css('margin-top', h+'px');
                    $('#promotionPieces_'+color).show();
                }

                function onDrop(source, target) {
                    // see if the move is legal
                    var piece = game.get(source);
                    var move = game.move({
                        from: source,
                        to: target,
                        promotion: 'q' // NOTE: always promote to a queen for example simplicity
                    });

                    // illegal move
                    if (move === null) {
                        return 'snapback';
                    }
                    
                    var promote = '';
                    if(piece.type == 'p' && (target[1] == '8' || target[1] == '1')) {
                        if($('#promoteToQueen').is(':checked')) {
                            promote = 'q';
                        } else {
                            game.undo();
                            promotionMove(source, target, piece.color);
                            return;
                        }
                    }
                    
                    $.get("../api/command/move?args=" + source + target + promote, function () {
                    });
                    updateStatus();
                }

                // update the board position after the piece snap
                // for castling, en passant, pawn promotion
                function onSnapEnd() {
                    board.position(game.fen());
                }

                game = new Chess(fen);
                var config = {
                    draggable: true,
                    position: fen,
                    onDragStart: onDragStart,
                    onDrop: onDrop,
                    onSnapEnd: onSnapEnd,
		    pieceTheme: 'chessboardjs/img/chesspieces/wikipedia/{piece}.png'
                }
                board = Chessboard('board', config);
                $(window).resize(board.resize);
		downloadAndUpdateConfig();
                updateStatus();
});
setInterval(checkConnection, 1000);

})();
